/*******************************************************************************
 *
 *  Classlights.pde 
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * Lights class.  This class manages the Lights' dimmer value.
 * The MSP uses a PWM with a timer period of 1ms running at 1Mhz.
 * so, there are 1,000 counts per period.  The dimmer value is 
 * scaled from 0 - 1000 where is 0 is off and 1000 is full on.
 * Since LEDs don't give a linear light output when you linearly
 * change the input power a log formula is applied to help
 * compensate. 
 */
class classLights {   
  int mx, my;
  int TouchX, TouchY;
  int TouchWidth, TouchHeight;
  int DimmerLast, Dimmer, DimmerPosition;
  boolean DimmerUpdated = false;
  PImage LightBright, LightDim, DimmerImg, DimmerIndicatorImg;
  
  classLights(int imx, int imy) {
    mx = imx;
    my = imy;
  }
  
  void setup(){
    TouchX = mx + 43;
    TouchY = my + 56;
    TouchWidth = 82;
    TouchHeight = 167;
    LightDim = loadImage("Light_Bulb_Dim.jpg");
    LightBright = loadImage("Light_Bulb_Bright.jpg");
    DimmerImg = loadImage("Switch_Plate.jpg");
    DimmerIndicatorImg = loadImage("Red Ball 2.jpg");
    DimmerIndicatorImg.resize(60,0);
    DimmerPosition = 220;
  }
  
  void draw() {      
    //Draw Light Bulb and switch plate
    tint(255,255);    
    image(DimmerImg, mx, my);
    // Draw indicator    
    image(DimmerIndicatorImg, mx + (DimmerImg.width/2) - (DimmerIndicatorImg.width/2), DimmerPosition-(DimmerIndicatorImg.height/2));
    
    image(LightDim, mx + DimmerImg.width + 20, my);
    // The image is at 50% opacity.
    tint(255,(float((TouchHeight - (DimmerPosition - TouchY)) - DimmerIndicatorImg.height/2)/100.0)*255);    
    
    image(LightBright, mx + DimmerImg.width + 20, my);
    tint(255,255);
      
    // Test if the cursor is within the dimmer region and clicked 
    if ((mouseX > TouchX) && (mouseX < TouchX+TouchWidth) && 
        (mouseY > (TouchY)) && (mouseY < TouchY+TouchHeight) && 
        (mousePressed == true) ) 
    {      
      // Determine Value from mouse Y coord
      DimmerPosition = mouseY; 
      // Ensure that the indicator doesn't go beyond the limits
      if(DimmerPosition > TouchY+TouchHeight-(DimmerIndicatorImg.height/2+5))
      {
        DimmerPosition = TouchY+TouchHeight-(DimmerIndicatorImg.height/2+5);
      }
      if(DimmerPosition < TouchY+DimmerIndicatorImg.height/2+5)
      {
        DimmerPosition = TouchY+DimmerIndicatorImg.height/2+5;
      }
      
      GenerateDimmerFromPosition(); 
    }
    if(Dimmer != DimmerLast)
    {
      DimmerUpdated = true;
      DimmerLast = Dimmer;
    }
  }
  
  void GenerateDimmerFromPosition ()
  {
    Dimmer = int((float((TouchHeight - (DimmerPosition - TouchY)) - DimmerIndicatorImg.height/2)) * 1.65);
    // Determine % of full scale
    Dimmer = int((float(Dimmer) / float(TouchHeight)) * 100.0);
    // apply an log formula to give a more linear output response
    Dimmer = int(5 * exp(Dimmer * 0.0535)); 
  }
  
  void SetDimmerPosition(boolean OnOff)
  {
    if(OnOff == true)
    {
      DimmerPosition = TouchY+DimmerIndicatorImg.height/2+5;
    }
    else
    {
      DimmerPosition = TouchY+TouchHeight-(DimmerIndicatorImg.height/2+5);
    }
    GenerateDimmerFromPosition();  
  }
  
}
