/*******************************************************************************
 *
 *  classThermistor.pde 
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * Thermistor class.  This class calculates temperature based on the
 * ADCval measurements from the MSP430.  
 * To aid in the calculations a first order caclculation is performed
 * which was choosen between a range of 15C and 40C.  Outside of this 
 * range the thermistor looses accuracy due to the first order calculation.
 */
class classThermistor { 
  int UNITS_C = 0;
  int UNITS_F = 1;
  int units = UNITS_C;
  
  float SetPoint;
  int ADCLimitHigh, ADCLimitLow, ADCSetPoint;
  int THERMOSTAT_STATE_IDLE = 1;
  int THERMOSTAT_STATE_HEATING = 2;
  int THERMOSTAT_STATE_COOLING = 3;
  int THERMOSTAT_STATE_ALARM = 4;
  
  boolean SetPointUpdated = false;
  
  float ConvertToTemperature(float AdcVal)
  {
    float Numerator = 470000*(AdcVal/1024);
    float Denominator = 1-(AdcVal/1024);
    float R1 = Numerator/Denominator;
    
    float TemperatureC = 66.12 * pow(0.99999042, R1);
    if( units == UNITS_C )
    {
      return(TemperatureC);
    }
    else
    {      
      //Tf = Tc/(5/9)+32
      float TemperatureF = (TemperatureC/(5.0/9.0))+32.0;
      return(TemperatureF);
    }    
  }
  
  void SetUnits(int val)
  {
    units = val;
  }
  
  float ConvertToR(float Temperature, float Offset)
  {        
    float R;
    if( units == UNITS_C )
    {
      R = log((Temperature+Offset)/66.12) / log(0.99999042);
    }
    else
    {
      // First convert F to C
      //Tc = (Tf-32) * (5/9)
      float TemperatureC = (Temperature - 32.0) * (5.0/9.0);      
      R = log((TemperatureC + Offset)/66.12) / log(0.99999042);      
      
    }
    return(R);
  }
  
  void setSetPointAndLimits(float temperatureADC)
  {
    SetPoint = temperatureADC;
    float RHigh = ConvertToR(temperatureADC,  3.0);
    float RLow =  ConvertToR(temperatureADC, -3.0);
    float RSetPoint = ConvertToR(temperatureADC, 0);
    
    ADCLimitHigh = int((RHigh/(RHigh+470000))*1024);
    ADCLimitLow = int((RLow/(RLow+470000))*1024);    
    ADCSetPoint = int((RSetPoint/(RSetPoint+470000))*1024);  
  }
  
} 
