/*******************************************************************************
 *
 *  classThermostat.pde 
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * Thermostat class.  Extends the Thermistor class to add displaying
 * and controling the setpoint information
 */
class classThermostat extends classThermistor { 
  int mx, my;
  float temperature;
  PImage ThermometerImg, ArrowImg, DegCImg, DegFImg, RedStripImg, MiniscusImg, ThermLineImg;
  PFont f;
  int SetPointBoxX, SetPointBoxY;
  int SetPointBoxWidth, SetPointBoxHeight;
  int SetPointlowLimit, SetPointHighLimit;
  int state;
  
  classThermostat(int imx, int imy) {
    mx = imx;
    my = imy;
    SetPointBoxX = mx+13;
    SetPointBoxY = my-30;
    SetPointBoxWidth = 55;
    SetPointBoxHeight = 50;
  }
  
  void setup(){
    f = loadFont("ComicSansMS-48.vlw"); 
    ArrowImg = loadImage("Arrow.jpg");
    ThermometerImg = loadImage("Thermometer.jpg");
    DegCImg = loadImage("Deg C.jpg");
    DegFImg = loadImage("Deg F.jpg");
    RedStripImg = loadImage("Thermometer Red Strip.jpg");
    MiniscusImg = loadImage("Thermometer Red Top Strip.jpg");
    ThermLineImg = loadImage("Thermometer Line.png");
    // default set to room temp
    if(this.units == this.UNITS_C)
    {
      temperature = 24.5;
      this.setSetPointAndLimits(24.5);   
    }
    else
    {
      temperature = 85;
      this.setSetPointAndLimits(85); 
    } 
    state = THERMOSTAT_STATE_IDLE;
  }
  void draw() {
    // Draw Thermometer
    image(ThermometerImg, mx, my);
    if(this.units == this.UNITS_C)
    {
      image(DegCImg, mx+(ThermometerImg.width/2)-(DegCImg.width/2)-5, my+235);
    }
    else
    {
      image(DegFImg, mx+(ThermometerImg.width/2)-(DegCImg.width/2)-5, my+235);
    }
    
    textFont(f,16);                 
    fill(255);              
    // Find Y of text based on temperature
    int Yoffset;
    if(this.units == this.UNITS_C)
    {
      Yoffset = 200 - int(((temperature-15.0) * 8.8)); 
    }
    else
    {
      Yoffset = 200 - int(((temperature-59.0) * 4.89)); 
    }    
    // Draw arrow
    image(ArrowImg,mx+75,my+Yoffset-5);
    // Draw remote Temperature
    text(str(temperature).substring(0,4), mx+75,my+Yoffset-5);      
    // Draw thermometer level
    int i;
    
    for(i=0;i<200-(Yoffset-MiniscusImg.height);i++)
    {
      image(RedStripImg,mx+21,my+200-i);
    }
    image(MiniscusImg,mx+21,my+200-i);
    tint(255,255);
    // Draw Set Temperature
    // Find Yoffset based on temperature
    if(this.units == this.UNITS_C)
    {
      Yoffset = 200 - int(((this.SetPoint-15.0) * 8.8)); 
    }
    else
    {
      Yoffset = 200 - int(((this.SetPoint-59.0) * 4.89));
    }
    
    // Draw Fadded box about limits
    fill(255,0,0,128);
    updateSetPointBox(Yoffset);
    
    // The Upper and lower levels are drawn as two rectangles
    rect(SetPointBoxX, SetPointBoxY + Yoffset, SetPointBoxWidth, 5);
    rect(SetPointBoxX, SetPointBoxY + Yoffset + SetPointBoxHeight + 10, SetPointBoxWidth, 5);
    // print setpoint temp text
    fill(255);
    text(str(this.SetPoint).substring(0,4), mx-20,SetPointBoxY+Yoffset+(SetPointBoxHeight/2)+15);
    // Print the Thermometer markers
    for(i=0; i<200; i+=20)
    {
      image(ThermLineImg,mx+21,my+200-i);
    }
  } 
  
  void updateSetPointBox(int Offset) {    
    if ((mouseX > SetPointBoxX) && (mouseX < SetPointBoxX+SetPointBoxWidth) && 
        (mouseY > (SetPointBoxY+Offset)) && (mouseY < SetPointBoxY+Offset+SetPointBoxHeight) && 
        (mousePressed == true) ) 
        {          
          int CenterOfBox = (mouseY-(SetPointBoxHeight/2));
          
          // update set point and Force Vertical limit by limiting set point
          if(this.units == this.UNITS_C)
          {
            this.SetPoint = (((220-CenterOfBox)/8.8)+15);
            if(this.SetPoint > 35)
            {
              this.SetPoint = 35;
            }
            if(this.SetPoint < 15)
            {
              this.SetPoint = 15;
            }
          }
          else
          {            
            this.SetPoint = (((220-CenterOfBox)/4.89)+59);            
            if(this.SetPoint > 95)
            {
              this.SetPoint = 95;
            }
            if(this.SetPoint < 59)
            {
              this.SetPoint = 59;
            }
          } 
          
          this.SetPointUpdated = true;
          setSetPointAndLimits(this.SetPoint);
        }
  }
}
