/*******************************************************************************
 *
 *  classTwitterCloud.pde 
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 /**
 * Twitter class.  Uses Twitter4J java library found here: 
 * http://twitter4j.org/en/index.html
 * On start up the user account is authorized and then 2 APIs are used.
 * Status updates are used when the temperature goes out of specifed range
 * and an alarm condition is genereated. The second API call is that this class
 * will poll the current status, and if changed since last time read, will return 
 * the message. This message is parsed in the main application and used as 
 * an input interface to set the se-temp and light dimmer values. 
 */
import twitter4j.conf.*;
import twitter4j.internal.async.*;
import twitter4j.internal.org.json.*;
import twitter4j.internal.logging.*;
import twitter4j.json.*;
import twitter4j.internal.util.*;
import twitter4j.management.*;
import twitter4j.auth.*;
import twitter4j.api.*;
import twitter4j.util.*;
import twitter4j.internal.http.*;
import twitter4j.*;
import twitter4j.internal.json.*;

// Time in milliseconds to fade cloud image in and out 
float DISPLAY_PERSISTANCE   =  10000.0;

TwitterFactory tf;
long LastMsgId;
PImage CloudImg;

class classTwitterCloud {
  int mx, my;
  long LastMessageId;
  String CurrentTwitterMessage;
  float CurrentMilliSeconds, PreviousMilliSeconds;
  float PersistenceMilliSeconds;
  float StartMillis = 0;
  String ConsumerKey;
  String ConsumerSecret;
  String oauth_token;
  String oauth_token_secret;
  String TwitterUserName;
  
  classTwitterCloud(int imx, int imy, String[] Acc) {
    mx = imx;
    my = imy;
    ConsumerKey = Acc[0];
    ConsumerSecret = Acc[1];
    oauth_token = Acc[2];
    oauth_token_secret = Acc[3];
    TwitterUserName = Acc[4];
  }
  
  void setup() {
    ConfigurationBuilder cb = new ConfigurationBuilder();
    cb.setDebugEnabled(true)
      .setOAuthConsumerKey(ConsumerKey)
      .setOAuthConsumerSecret(ConsumerSecret)
      .setOAuthAccessToken(oauth_token)
      .setOAuthAccessTokenSecret(oauth_token_secret);
    tf = new TwitterFactory(cb.build());
        
    Twitter twitter = tf.getInstance();
    CloudImg = loadImage("Message_Cloud.jpg");
  }
  
  void draw() {
    if(PersistenceMilliSeconds != 0)
    {
      CurrentMilliSeconds = millis(); 
      if(CurrentMilliSeconds != PreviousMilliSeconds)
      {
        PersistenceMilliSeconds = PersistenceMilliSeconds - (CurrentMilliSeconds - PreviousMilliSeconds);
        PreviousMilliSeconds = CurrentMilliSeconds;         
      }
      float FadeInTime = millis() - StartMillis;
      float FadeOutTime = (StartMillis + DISPLAY_PERSISTANCE) - millis();
      if(FadeInTime < 2000.0)
      {
        // Fade in
        tint(255,FadeInTime/2000.0*255);
        fill(200,200,200, (FadeInTime/2000.0)*255);
      }      
      else if(FadeOutTime < 3000.0)
      {
        // Fade in
        tint(255,((FadeOutTime-1000.0)/2000.0)*255);
        fill(200,200,200, ((FadeOutTime-1000.0)/2000.0)*255);
      }     
      else
      {
        fill(200,200,200, 255);
        tint(255,255);
      } 
      // Draw Cloud
      image(CloudImg, mx, my); 
      // Draw text      
      textAlign(CENTER);
      text(CurrentTwitterMessage, mx+(CloudImg.width/2)-80, my+(CloudImg.height/2)-50, 175, 130);
      textAlign(LEFT);
      tint(255,255);
    }    
  }
  
  void PrintTwitterMessage (String message)
  {
    CurrentTwitterMessage = message;
    PersistenceMilliSeconds = DISPLAY_PERSISTANCE; 
    StartMillis = millis();    
  }
  
  void UpdateStatus(String StrStatus) {
    println("Tweet Tweet");
    Twitter twitter = tf.getInstance();
    try {
      Status st = twitter.updateStatus(StrStatus + " " + getDateTime());
      println(st);
    }
    catch (TwitterException e) {
      println(e.getStatusCode());
    }
  }
  
  void SendPrivateMessage(String StrStatus) {
    try {
      Twitter twitter = tf.getInstance();      
      User user = twitter.showUser(TwitterUserName);    
      DirectMessage message = twitter.sendDirectMessage(user.getStatus().getId(), StrStatus);
    }
    catch (TwitterException e) {
      println(e.getStatusCode());
    }
  }

  String CheckForTwitterMessage() {    
    try {
      Twitter twitter = tf.getInstance();
      User user = twitter.showUser(TwitterUserName);
      Status StrStatus = user.getStatus();  
      /* We fetch our status update and then retrieve the unique message ID.  From this
         we can see if we have processed this or not.  If not then we then service this as
         a new status update. */
      println(StrStatus);
      if(StrStatus != null)
      {
        if(user.getStatus().getId() != LastMessageId)
        {
          // Update the ID
          LastMessageId = user.getStatus().getId();
          return (user.getStatus().getText());
        }
      }
      //println("@" + user.getScreenName() + " - " + user.getStatus().getId() + " - " + user.getStatus().getText());
             
    }
    catch (TwitterException e) {
      println(e.getStatusCode());
    }
    return (null);
  }
  
  String getDateTime() {
    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
    Date date = new Date();
    return dateFormat.format(date);
  }

}
